package edu.harding.tictactoe;

import android.app.Activity;
import android.graphics.Color;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

public class AndroidTicTacToeActivity extends Activity {
    
	// Indicates if game is currently over or not
	private boolean mGameOver = false;
	
	// Buttons making up the board
	private Button mBoardButtons[];
	
	// Whose turn to go first
	private char mTurn = TicTacToeGame.COMPUTER_PLAYER;    
	
	// Keep track of wins
	private int mHumanWins = 0;
	private int mComputerWins = 0;
	private int mTies = 0;
	
	// Represents the internal state of the game
	private TicTacToeGame mGame;
	
	// Various text displayed
	private TextView mInfoTextView; 
	private TextView mHumanScoreTextView;
	private TextView mComputerScoreTextView;
	private TextView mTieScoreTextView;
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        mBoardButtons = new Button[TicTacToeGame.BOARD_SIZE];
        mBoardButtons[0] = (Button) findViewById(R.id.one);
        mBoardButtons[1] = (Button) findViewById(R.id.two); 
        mBoardButtons[2] = (Button) findViewById(R.id.three); 
        mBoardButtons[3] = (Button) findViewById(R.id.four); 
        mBoardButtons[4] = (Button) findViewById(R.id.five); 
        mBoardButtons[5] = (Button) findViewById(R.id.six); 
        mBoardButtons[6] = (Button) findViewById(R.id.seven); 
        mBoardButtons[7] = (Button) findViewById(R.id.eight); 
        mBoardButtons[8] = (Button) findViewById(R.id.nine); 
        
        mInfoTextView = (TextView) findViewById(R.id.information); 
        mHumanScoreTextView = (TextView) findViewById(R.id.player_score);
        mComputerScoreTextView = (TextView) findViewById(R.id.computer_score);
        mTieScoreTextView = (TextView) findViewById(R.id.tie_score);
        
        mGame = new TicTacToeGame();
        
        startNewGame();
    }
    
    @Override 
    public boolean onCreateOptionsMenu(Menu menu) { 
         super.onCreateOptionsMenu(menu); 
         menu.add("New Game");          	    
         return true;
    } 
    
    
    // Handles menu item selections 
    public boolean onOptionsItemSelected(MenuItem item) {
        startNewGame();
        return true;
    }   
    
    
    // Set up the game board. 
    private void startNewGame() {   	
    	
    	mGame.clearBoard();
    	
    	// Reset all buttons
    	for (int i = 0; i < mBoardButtons.length; i++) {
    		mBoardButtons[i].setText("");
    		mBoardButtons[i].setEnabled(true);   
    		mBoardButtons[i].setOnClickListener(new ButtonClickListener(i));    		 		   
    	}

    	// Alternate who goes first
    	if (mTurn == TicTacToeGame.HUMAN_PLAYER) {
    		mTurn = TicTacToeGame.COMPUTER_PLAYER;
    		mInfoTextView.setText(R.string.first_computer);
    		int move = mGame.getComputerMove();
    		setMove(TicTacToeGame.COMPUTER_PLAYER, move);
    		mInfoTextView.setText(R.string.turn_human);
    	}
    	else {
    		mTurn = TicTacToeGame.HUMAN_PLAYER;
    		mInfoTextView.setText(R.string.first_human); 
    	}	
    	
    	mGameOver = false;
    } 
    
    private void setMove(char player, int location) {
    	
    	mGame.setMove(player, location);
    	mBoardButtons[location].setEnabled(false); 
    	mBoardButtons[location].setText(String.valueOf(player));
    	if (player == TicTacToeGame.HUMAN_PLAYER) 
    		mBoardButtons[location].setTextColor(Color.rgb(0, 200, 0));     	    
    	else 
        	mBoardButtons[location].setTextColor(Color.rgb(200, 0, 0)); 
    }
    
    // Handles clicks on the game board buttons
    private class ButtonClickListener implements View.OnClickListener { 
        int location; 

        public ButtonClickListener(int location) { 
             this.location = location; 
        } 

        public void onClick(View view) { 
        	if (!mGameOver && mBoardButtons[location].isEnabled()) {
        		setMove(TicTacToeGame.HUMAN_PLAYER, location);        		
            	
            	// If no winner yet, let the computer make a move
            	int winner = mGame.checkForWinner();
            	if (winner == 0) { 
            		mInfoTextView.setText(R.string.turn_computer); 
            		int move = mGame.getComputerMove();
            		setMove(TicTacToeGame.COMPUTER_PLAYER, move);
            		winner = mGame.checkForWinner();
            	} 
            	
            	if (winner == 0)
            		mInfoTextView.setText(R.string.turn_human);
            	else {        
            		// Game is over
	            	if (winner == 1) {
	            		mTies++;
	            		mTieScoreTextView.setText(Integer.toString(mTies));
	            		mInfoTextView.setText(R.string.result_tie); 
	            	}
	            	else if (winner == 2) {
	            		mHumanWins++;
	            		mHumanScoreTextView.setText(Integer.toString(mHumanWins));
	            		mInfoTextView.setText(R.string.result_human_wins);
	            	}
	            	else if (winner == 3) {
	            		mComputerWins++;
	            		mComputerScoreTextView.setText(Integer.toString(mComputerWins));
	            		mInfoTextView.setText(R.string.result_computer_wins);
	            	}
	            	
	            	mGameOver = true;
            	}
             } 
        } 
   }    
}